/*
 *
 * Written By:   Philip Isenhour
 * Date:         October 1996
 * Description:  Implementation of (server-side) room data.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.4 1997/01/07 20:52:58 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package vt.cs.virtSchool.server;

import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.util.*;
import SharedData.Session;
//import SharedData.SocketSession; // pre 0.90 JSDA release
import SharedData.SessionFactory;
import SharedData.Client;
import SharedData.ClientEvent;
import SharedData.Channel;
import SharedData.Data;
import SharedData.SharedDataException;
import vt.cs.db.*;

/**
 * Implementation of (server-side) room data. This object stores configuration
 * data as well as the current state of the activities on the table and
 * shelf. Eventually it should probably also listen to the chat channel
 * and log that data as well.
 *
 * @see vt.cs.virtSchool.server.RoomData
 * @version $Id Exp $
 */
public class RoomDataImpl extends UnicastRemoteObject 
implements RoomData, Client {
  private String shortname = "UnnamedRoom";
  private String longname = "(Unnamed Room)";
  private String descriptionURL = null;
  private String hostname = null;
  private int port = 0;
  private String sessionURL = null;
  private String sessionURLbase = null;
  private String objectURL = null;

  private Session session = null;
  private Session clientSession = null;

  private Channel shelfChannel = null;
  private ActivityChannelConsumer shelfConsumer = null;
  private Channel tableChannel = null;
  private ActivityChannelConsumer tableConsumer = null;

  private Channel chatChannel = null;
  private Channel wbChannel = null;

  /* 
   * Storage for activities and links present in the room.
   */
  private	ActivityTable shelfActivities	= null;
  private	ActivityTable tableActivities	= null;

  public RoomDataImpl(String shortname, String longname, 
		      String descriptionURL, String hostname, int port) 
    throws RemoteException {

      super();

      debug("Creating Room shortname="+shortname);
      this.longname = longname;
      this.shortname = shortname;
      this.hostname = hostname;
      this.descriptionURL = descriptionURL;
      this.port = port;

      this.shelfActivities = new ActivityTable();
      this.tableActivities = new ActivityTable();

      // sessionURL = "coll://" + hostname + ":" + port + "/Session/" + shortname;
      sessionURLbase ="coll://" + hostname + ":" + port + "/" +
	WorldServer.sessionType+ "/Session/";
      sessionURL = sessionURLbase + shortname;

      sessionURLbase ="coll://" + hostname + ":" + (port+3) + "/" +
	WorldServer.sessionType+ "/Session/";

      objectURL = "rmi://" + hostname + "/" + RoomData.ROOMTAG + "/" + 
	shortname;

      try {
	//session = SessionFactory.createSession(sessionURL);  // JSDA .90

	//session = new SharedData.socket.socketSession(shortname);
	//SharedData.Naming.rebind(sessionURL, session);

	clientSession = SessionFactory.createSession(sessionURL);
	debug("session created: "+sessionURL);
	clientSession.join(this);
	debug("joined session "+sessionURL);

	/*
	 * Create channels for activity (and link) events, 
	 * text chat, and whiteboard.
	 */

	clientSession.createChannel(RoomData.TABLE_CHANNEL, true);
	debug("created channel "+RoomData.TABLE_CHANNEL+" on session " +sessionURL);
	tableChannel = Channel.join(clientSession,
				    RoomData.TABLE_CHANNEL, this);
	debug("joined channel "+RoomData.TABLE_CHANNEL+" on session "+sessionURL);
	tableConsumer = new ActivityChannelConsumer(this, tableChannel,
						    tableActivities);
	clientSession.createChannel(RoomData.SHELF_CHANNEL, true);
	shelfChannel = Channel.join(clientSession,
				    RoomData.SHELF_CHANNEL, this);
	shelfConsumer = new ActivityChannelConsumer(this, shelfChannel,
						    shelfActivities);

	clientSession.createChannel(RoomData.CHAT_CHANNEL, true);
	chatChannel = Channel.join(clientSession,
				   RoomData.CHAT_CHANNEL, this);
	clientSession.createChannel(RoomData.WHITEBOARD_CHANNEL, true);
	wbChannel = Channel.join(clientSession,
				 RoomData.WHITEBOARD_CHANNEL, this);

	debug("created channels on session "+sessionURL);
	/*
	 * Bind this object to an rmi url.
	 */
	java.rmi.Naming.rebind(objectURL, (Remote)this);
	debug("Bound RoomDataImpl to " + objectURL);
      }
      catch (Exception e) {
	e.printStackTrace();
      }
	debug("ending Construction of RoomDataImpl"+sessionURL);
  }

  /**
   * Returns the (human-readable) name for the room.
   */
  public String getRoomName() throws RemoteException {
    return longname;
  }

  /**
   * Returns the (JSDA) URL for the this room's session.
   */
  public String getSessionURL() throws RemoteException {
    return sessionURL;
  }

  /**
   * Returns the (JSDA) base URL (sessionURL - name); 
   */
  public String getSessionURLbase() throws RemoteException {
    return sessionURLbase;
  }

  /**
   * Returns the description URL specified in the config file.
   */
  public String getDescriptionURL() throws RemoteException {
    return descriptionURL;
  }

  public Hashtable getShelfActivities() throws RemoteException {
    return shelfActivities;	
  }

  public Hashtable getTableActivities() throws RemoteException {
    return tableActivities;	
  }

  private Hashtable sessions = new Hashtable();
  /**
   * Opens and holds a session named by on sessionURL
   */
  public void holdSession(String sessionURL) throws RemoteException {
    debug("holdSession: "+sessionURL);
    try {
      Session session = SessionFactory.createSession(sessionURL);
      session.join(this);
      sessions.put(sessionURL, session);
    } catch (Exception ex) {
      System.out.println("Error creating session: "+sessionURL+
			 " Exception: "+ex);
      ex.printStackTrace();
    }
  }

  public void closeSession(String sessionURL) throws RemoteException {
    Session session = (Session)sessions.remove(sessionURL);
    if (session != null) {
      try {
	session.leave(this);
      } catch (Exception ex) {
	System.out.println("Error leaving session: "+sessionURL+
			   " Exception: "+ex);
	ex.printStackTrace();
      }
    }
  }

  /** holdSession must be called first.
   * This method is not used.
   */
  public void holdChannel(String sessionURL, String channelName) 
    throws RemoteException {
      debug("holdChannel: "+sessionURL+" : "+channelName);
      Session session = (Session)sessions.get(sessionURL);
      if (session == null) {
	// call holdSession first
	System.out.println("Session not found: "+sessionURL +
			   " (call holdSession() first)");
	return;
      }
      try {
	session.createChannel(channelName, true);
	debug("Channel successfully created: "+channelName);
	Channel tempChannel = Channel.join(session, channelName, this);
	debug("Channel successfully joined: "+channelName);

      } catch (Exception sdex) {
	// this channel may already exist.  If so, great.
	if (sdex instanceof SharedDataException){
	  if (((SharedDataException)sdex).getType() 
	      != SharedDataException.CHANNEL_EXISTS){
	    sdex.printStackTrace();
	  }
	}
      }
      // We should hold the channelName in a hashtable of channels
      // per session (Hashtable sessionChannels) so that we can look 
      // it up for a closeChannel() call
  }

  public void closeChannel(String sessionURL, String channelName) 
    throws RemoteException {
  }

  /* ----------- Client Interface methods ----------- */
  public String authenticate(String challenge) {
    debug("authenticate called challenge = "+challenge);
    return null;
  }
  public void clientChanged(ClientEvent event) {
    debug("clientChanged "+event);
  }
  public String getName() {
    String returnString = "_" + longname + " (RoomData Object in Server)";
    debug("getName: "+returnString);
    return returnString;
  }
  /* ------------------------------------------------ */


  /**
   * This method returns a unique client name based on the name it
   * is passed. (We just append an appropriate digit to the name.)
   * This was added as a work-around for
   * a JSDA glitch which (I think) has been fixed.
   */
  public String getUniqueClientName(String name) throws RemoteException {
    String	newName	= null;
		
    newName = name.concat("." + System.currentTimeMillis()%10000);
    return newName;
  }

  public String getDatabaseSetName() throws RemoteException {
    return "rmidb:rmi://" + hostname;
  }

  /**
   * Called when a client wants to launch an activity applet.
   * Eventually we will want to send a message to all clients when
   * this occurs. (eg, if we have icons representing users, they
   * may want to draw links from the activity icons to the user
   * icons.)
   */
  public boolean permissionToLaunchActivity(Activity act, String user)
    throws RemoteException {
      boolean	permission	= false;
      synchronized(tableActivities) {
	Activity	a = (Activity) tableActivities.get(act.id());
	if ((a.executedBy == null) || (a.executedBy.equals(user))) {
	  a.executedBy = user;
	  permission = true;
	}
      }
      return permission;
  }

  public void activityClosed(Activity act, String user) 
    throws RemoteException {
      synchronized(tableActivities) {
	Activity    a = (Activity) tableActivities.get(act.id());
	if ((a.executedBy != null) && (a.executedBy.equals(user))) {
	  a.executedBy = null;			
	}
      }
  }

  /**
   * Used at startup to add activities to the shelf. Currently this also
   * adds the room's URL to the activity's parameters.
   */
  public void addShelfActivity(Activity act) {
    act.addURLParameter("RoomURL", objectURL);
    shelfActivities.put(act.id(), act);
  }

	private static final boolean debug = false;
	private void debug(String msg) {
	  if (debug)
	    System.out.println(getClass().getName()+": "+msg);
	}

}

class ActivityTable extends Hashtable implements ActivityContainer {
	
	/* ---------- ActivityContainer Interface Methods ---------- */
    public void activityAdded(Activity act){
		put(act.id(), act);
	}
    public void activityMoved(Activity act){
		remove(act.id());
		put(act.id(), act);
	}
	public void activityRemoved(Activity act){}
    public void activityClaimed(Activity act){}
    public void activityReleased(Activity act){}
    public void linkAdded(Link link){
		put(link.id(), link);
	}
    public void linkRemoved(Link link){
		remove(link.id());
	}
	/* --------------------------------------------------------- */

	private static final boolean debug = false;
	private void debug(String msg) {
	  if (debug)
	    System.out.println(getClass().getName()+": "+msg);
	}
}


