/*
 *
 * Written By:   Philip Isenhour
 * Date:         11 September 1996
 * Description:  Interface for producer->consumer data stream functionality.
 *
 * Usage Notes:
 *
 */

package vt.cs.virtSchool.util;

import java.util.*;
import java.applet.Applet;

import vt.cs.virtSchool.util.*;
import vt.cs.virtSchool.objectManager.ObjectManager;

/**
 * ProducerStream is the producer-side representation of a producer-consumer
 * data stream. Producers may specify a human-readable label, human-readable
 * unit, minimum expected value, and maximum expected value for the stream.
 * (Label and type information is mandatory, other data is optional but
 * should be provided if possible.) Eventually, the producer will also
 * be able to buffer data and control when it is sent to the consumers.
 */
public class ProducerStream {
	private boolean		streamIsBuffered = false;
	private String 		appletName = null;
	private String 		label = null;
	private String 		type = null;
	private String 		unit = null;
	private Object		min = null;
	private Object		max = null;

	// Initially we'll use the existing DataSet implemenation for this...
    private ObjectManager   objectManager = null;
    private RemoteObject    remoteObjUtil = null;
    private DataSet         dataSet = null;

	void init(Applet app, String label, String type) {
		remoteObjUtil = new RemoteObject();
		objectManager = remoteObjUtil.getObjectManager(app);
		dataSet = remoteObjUtil.getDataSet(app, objectManager);

		this.appletName = app.toString();
		this.label = label;
		this.type = type;
	}

	/**
	 * Constructs a ProducerStream for which unit and range (min/max)
	 * information is either not known or not applicable.
	 *
	 * @param applet The producer applet writing data to this stream
	 * @param label Human-readable string describing this stream
	 * @param type Type of data contained in this stream. Must be fully qualified, eg, <TT>java.lang.Long</TT>. Only classes (not base types) should be used.
	 */
	public ProducerStream (Applet applet, String label, String type) {
		init(applet, label, type);
	}

	/**
	 * Constructs a ProducerStream for which unit information is known,
	 * but for which range (min/max) information is either not known or 
	 * not applicable.
	 *
	 * @param applet The producer applet writing data to this stream
	 * @param label Human-readable string describing this stream
	 * @param type Type of data contained in this stream. Must be fully qualified, eg, <TT>java.lang.Long</TT>. Only classes (not base types) should be used.
	 * @param unit Unit of measurement (eg, meters)
	 */
	public ProducerStream (Applet applet, String label, String type, 
		String unit) {
		this.unit = unit;
		init(applet, label, type);
	}

	/**
	 * Constructs a ProducerStream for which unit and range information is 
	 * known. This will eventually throw some sort of exception if the
	 * min and max values are not of the specified type.
	 *
	 * @param applet The producer applet writing data to this stream
	 * @param label Human-readable string describing this stream
	 * @param type Type of data contained in this stream. Must be fully qualified, eg, <TT>java.lang.Long</TT>. Only classes (not base types) should be used.
	 * @param unit Unit of measurement (eg, meters)
	 * @param min Minimum value expected in the stream
	 * @param max Maximum value expected in the stream
	 */
	public ProducerStream (Applet applet, String label, String type, 
		String unit, Object min, Object max) {
		this.unit = unit;
		this.min = min;
		this.max = max;
		init(applet, label, type);
	}

	/**
	 * Sets the unit of the stream to the specified value. Eventually
	 * it would be nice if consumers could be notified of these changes,
	 * but that is not currently done.
	 */
	public void setUnit(String unit) {
		this.unit = unit;
		// Need to notify consumers of change?
	}

	/**
	 * Sets the expected range of the stream to the specified value. 
	 * These values are meant to provide consumers with information on
	 * how to present the data and are <I>not</I> enforced in any way.
	 * Will eventually throw some sort of exception if the range values
	 * are not of the correct type. Eventually
	 * it would be nice if consumers could be notified of these changes,
	 * but that is not currently done.
	 */
	public void setRange(Object min, Object max) {
		this.min = min;
		this.max = max;
		// Need to notify consumers of change?
	}

	/**
	 * Specifies that the stream should (or should not) be buffered. Data
	 * written to a buffered stream will not be sent until the producer
	 * calls the flush() method. <I>Not yet implemented</I>
	 */
	public void setBuffered(boolean isBuffered) {
		streamIsBuffered = isBuffered;
	}

	/**
	 * Returns true if the stream has been set to buffered.
	 */
	public boolean isBuffered() {
		return streamIsBuffered;
	}

	/**
	 * Writes a value to the stream with the specified timestamp. The 
	 * producer may decide how this timestamp is generated.
	 */
	public void write(Object value, long time) {
		// For now, ignore applet name
		try {
			dataSet.put(label, new DataObject(value, time));
		}
		catch (java.rmi.RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Writes a value to the stream. A timestamp value will be provided.
	 */
	public void write(Object value) {
		// If the user doesn't provide a time value, generate one here

		write(value, System.currentTimeMillis());
	}

	/**
	 * Sends any buffered data to the consumers. <I>Not yet implemented</I>
	 */
	public void flush() {
	}

	/**
	 * Clears all data in the stream. Will not clear data previously stored
	 * by consumers. <I>Not yet implemented</I>
	 */
	public void clear() {
	}
}




